package edu.gatech.cercs.soa.core;


import java.io.*;
import java.util.Date;

import edu.gatech.cercs.soa.core.fpq.IFlightPricingQuery;
import edu.gatech.cercs.soa.core.fpq.IHeader;
import edu.gatech.cercs.soa.core.fpr.IFlightPricingResult;
import edu.gatech.cercs.soa.core.fpr.IHeaderResult;
import edu.gatech.cercs.soa.core.xml.XMLProxy;

/**
 * The <code>FlightPricingUtils</code> class contains a collection of static
 * utility methods that are often used in various projects.
 * 
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class FlightPricingUtils {

   // For generating unique query Ids.
   private static Date queryTimestamp;
   private static int queryCount;

   static {
      queryTimestamp = new Date();
      queryCount = 0;
   }
   
   /**
    * Generate a unique id for queries using timestamp and a counter.
    * @return String - date time formatted.
    */
   public static synchronized String createUniqueQueryId() {
      Date ts = new Date();
      if (queryTimestamp.equals(ts)) {
          queryCount++;
      }
      else {
          queryTimestamp = ts;
          queryCount = 0;
      }
      return DateUtils.formatDefault(queryTimestamp) + " " + queryCount;
   }

   /**
    * Parse the input xml string to a new flight pricing query object.
    * @param xml - a non-null string
    * @return IFlightPricingQuery object
    * @throws IOException
    */
   public static IFlightPricingQuery parseQuery(String xml) throws IOException {
      IFlightPricingQuery fpQuery = FlightPricingFactory.createQuery();
      BufferedReader reader = null;
      try {
         reader = new BufferedReader(new InputStreamReader(
                  new ByteArrayInputStream(xml.getBytes())));
         (new XMLProxy(fpQuery)).parse(reader);
      }
      finally {
         if (reader != null)
            reader.close();
      }
      return fpQuery;
   }

   /**
    * Parse the input xml string to a new flight pricing result object.
    * @param xml - a non-null string
    * @return IFlightPricingResult object
    * @throws IOException
    */
   public static IFlightPricingResult parseResult(String xml)
            throws IOException {
      IFlightPricingResult fpResult = FlightPricingFactory.createResult();
      BufferedReader reader = null;
      try {
         reader = new BufferedReader(new InputStreamReader(
                  new ByteArrayInputStream(xml.getBytes())));
         (new XMLProxy(fpResult)).parse(reader);
      }
      finally {
         if (reader != null)
            reader.close();
      }
      return fpResult;
   }

   /**
    * Print the contents of a flight pricing query object to a String.
    * @param fpr - a non-null flight pricing query object
    * @return String - a non-null xml string
    */
   public static String dumpQuery(IFlightPricingQuery fpq) {
      StringWriter sw = new StringWriter();
      PrintWriter pw = new PrintWriter(sw);
      fpq.saveAsXML(pw);
      pw.flush();
      pw.close();
      return sw.toString();
   }

   /**
    * Print the contents of a flight pricing result object to a String.
    * @param fpr - a non-null flight pricing result object
    * @return String - a non-null xml string
    */
   public static String dumpResult(IFlightPricingResult fpr) {
      StringWriter sw = new StringWriter();
      PrintWriter pw = new PrintWriter(sw);
      fpr.saveAsXML(pw);
      pw.flush();
      pw.close();
      return sw.toString();
   }

   /**
    * Copies all the attributes from the flight pricing query header to the
    * flight pricing result header object, except the timestamp attribute.
    * @param fpq - a non-null flight pricing query object.
    * @param fpr - a non-null flight pricing result object.
    */
   public static void updateResultHeader(IFlightPricingQuery fpq,
            IFlightPricingResult fpr) {
      updateResultHeader(fpq.getHeader(), fpr.getHeader());
   }

   /**
    * Copies all the attributes from the flight pricing query header to the
    * flight pricing result header object, except the timestamp attribute.
    * @param h - a non-null flight pricing query header object.
    * @param fpr - a non-null flight pricing result object.
    */
   public static void updateResultHeader(IHeader h, IFlightPricingResult fpr) {
      updateResultHeader(h, fpr.getHeader());
   }
   
   /**
    * Copies all the attributes from the flight pricing query header to the
    * flight pricing result header object, except the timestamp attribute.
    * @param h - a non-null flight pricing query header object.
    * @param hr - a non-null flight pricing result header object.
    */
   public static void updateResultHeader(IHeader h, IHeaderResult hr) {
      hr.setCustomerId(h.getCustomerId());
      hr.setQueryId(h.getQueryId());
      hr.setQueryMode(h.getQueryMode());
      hr.setSearchId(h.getSearchId());
      // hr.setTimestamp(h.getTimestamp()); Note: Read-only
      hr.setExpiration(h.getExpiration());
   }

   /**
    * Parses the input xml string and returns the header part of the flight
    * pricing query.
    * @param queryXml - a non-null flight pricing query xml string
    * @return IHeader
    */
   public static IHeader parseQueryHeader(String queryXml) {
      IHeader hdr = null;
      if (queryXml != null && queryXml.indexOf("<Header>") > 0) {
         String headerXml = queryXml.substring(queryXml.indexOf("<Header>"),
                  queryXml.indexOf("</Header>") + "</Header>".length());
         hdr = FlightPricingFactory.createQueryHeader();
         BufferedReader reader = null;
         try {
            reader = new BufferedReader(new InputStreamReader(
                     new ByteArrayInputStream(headerXml.getBytes())));
            (new XMLProxy(hdr)).parse(reader);
         }
         catch (IOException ioe) {
            ioe.printStackTrace(); // TODO - log the error
         }
         finally {
            try {
               if (reader != null)
                  reader.close();
            }
            catch (IOException ioe) {
            }
         }
      }
      return hdr;
   }

   /**
    * Parses the input xml string and returns the header part of the
    * flight pricing result.
    * @param resultXml - a non-null flight pricing result xml string
    * @return IHeaderResult
    */
   public static IHeaderResult parseResultHeader(String resultXml) {
      IHeaderResult hdr = null;
      if (resultXml != null && resultXml.indexOf("<Header>") > 0) {
         String headerXml = resultXml.substring(resultXml.indexOf("<Header>"),
                  resultXml.indexOf("</Header>") + "</Header>".length());
         hdr = FlightPricingFactory.createHeaderResult();
         BufferedReader reader = null;
         try {
            reader = new BufferedReader(new InputStreamReader(
                     new ByteArrayInputStream(headerXml.getBytes())));
            (new XMLProxy(hdr)).parse(reader);
         }
         catch (IOException ioe) {
            ioe.printStackTrace(); // TODO - log the error
         }
         finally {
            try {
               if (reader != null)
                  reader.close();
            }
            catch (IOException ioe) {
            }
         }
      }
      return hdr;
   }

}
