package edu.gatech.cercs.soa.gds.client;

import java.io.File;
import java.util.Hashtable;

import javax.jms.*;
import javax.naming.*;

import org.activemq.ActiveMQConnectionFactory;

import edu.gatech.cercs.soa.core.FileUtils;
import edu.gatech.cercs.soa.core.PropertyLoader;


/**
 * Describe <code>FlightPricingMessageClient</code> here.
 *
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class FlightPricingMessageClient {
   
   private static final String NAMING_FACTORY = "java.naming.factory.initial";
   private static final String NAMING_PROVIDER = "java.naming.provider.url";
   private static final String CONN_FACTORY_NAMES = "connectionFactoryNames";
   private static final String QUERY_QUEUE_NAME = "queue.FlightPricingQueryQueue";
   private static final String RESULT_QUEUE_NAME = "queue.FlightPricingResultQueue";
   
   private static final String PROPERTIES_FILE = "client.properties";
   
   private PropertyLoader propLoader;
   private String queryFileName;
   
   public FlightPricingMessageClient(String queryFileName) {
      this.queryFileName = queryFileName;
      this.propLoader = PropertyLoader.getInstance(PROPERTIES_FILE);
   }
   
   public void send() {
      String content = FileUtils.loadFileContent(queryFileName);
      
      QueueConnection conn = null;
      QueueSession session = null;
      MessageProducer producer = null;
      TextMessage flightPricingQuery = null;
      
      try {
         Hashtable<Object, Object> env = new Hashtable<Object, Object>();
         env.put(NAMING_FACTORY, propLoader.getValue(NAMING_FACTORY));
         env.put(NAMING_PROVIDER, propLoader.getValue(NAMING_PROVIDER));
         env.put(CONN_FACTORY_NAMES, propLoader.getValue(CONN_FACTORY_NAMES));
         env.put(QUERY_QUEUE_NAME, propLoader.getValue(QUERY_QUEUE_NAME));

         Context ctx = new InitialContext(env);
         QueueConnectionFactory factory = (QueueConnectionFactory)
            ctx.lookup(propLoader.getValue(CONN_FACTORY_NAMES));
         conn = factory.createQueueConnection();
         
         Queue myQueue = (Queue) ctx.lookup(propLoader.getValue(QUERY_QUEUE_NAME));
         session = conn.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
         producer = session.createProducer(myQueue);

         conn.start();
         flightPricingQuery = session.createTextMessage();
         flightPricingQuery.setText(content);
         
         producer.send(flightPricingQuery);
         System.out.println("Query Sent to " + QUERY_QUEUE_NAME + "!");
      } 
      catch (NamingException ne) {
         ne.printStackTrace();
      } 
      catch (JMSException e) {
         e.printStackTrace();
      }
      finally {
         try {
            if (producer != null) {
               producer.close();
            }
            if (session != null) {
               session.close();
            }
            if (conn!= null) {
               conn.close();
            }
         } 
         catch (JMSException e) {
            e.printStackTrace();
         }
      }
   }
   
   public void recv() {
      MessageConsumer consumer = null;
      QueueConnection connection = null;
      QueueSession session = null;
      
      try {
         System.out.println("Waiting on " + RESULT_QUEUE_NAME + "...");
         
         ActiveMQConnectionFactory connectionFactory = 
            new ActiveMQConnectionFactory(propLoader.getValue(NAMING_PROVIDER));
         connection = (QueueConnection)connectionFactory.createConnection();
         connection.start();
         
         session = connection.createQueueSession(false,Session.AUTO_ACKNOWLEDGE);
         Queue queue = session.createQueue(propLoader.getValue(RESULT_QUEUE_NAME));
         consumer = session.createConsumer(queue);
         
         Message message = consumer.receive();
         processMessage(message);
      } 
      catch (Exception ex) {
         ex.printStackTrace();
      }
      finally {
         try {
            if (consumer != null) {
               consumer.close();
            }
            if (session != null) {
               session.close();
            }
            if (connection!= null) {
               connection.close();
            }
         } 
         catch (JMSException e) {
            e.printStackTrace();
         }
      }
   }
   
   private void processMessage(Message message) {
      try {
         TextMessage request = (TextMessage) message;
         System.out.println("Received Flight Pricing Query Result: \n" + 
                  request.getText());
      } 
      catch (Exception ex) {
         ex.printStackTrace();
      }
   }
      
   public static void main(String[] args)  {
      
      String fileName = null;
      try {
         fileName = args[0];
      } 
      catch (Exception e) {
         System.out.println("Enter the Flight Pricing Query file to send.");
         return;
      }
      
      File f = new File(fileName);
      if (!f.exists() || !f.isFile()) {
         System.out.println(fileName + "is not a valid file name!");
         return;
      }
      
      FlightPricingMessageClient fpqClient = 
          new FlightPricingMessageClient(fileName);
      fpqClient.send();
      fpqClient.recv();
   }

}

