package edu.gatech.cercs.soa.gds;

import java.io.IOException;
import java.util.*;

import edu.gatech.cercs.soa.core.FlightPricingUtils;
import edu.gatech.cercs.soa.core.PropertyLoader;
import edu.gatech.cercs.soa.core.fpr.IFlightPricingResult;
import edu.gatech.cercs.soa.core.fpr.IHeaderResult;


/**
 * The FlightPricingState is a singleton class that maintains a cache of
 * recent flight pricing search results from the QPS service. 
 *
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class FlightPricingState {

    private static final String PROPERTIES_FILE = "gds.properties";
    private static final String GC_INTERVAL = "cache.FlightPricingGCInterval";
    
    private static FlightPricingState instance;
    
    //Map of {customerId,queryId} <-> {result xml}
    private static Map<String, String> fpResultMap; //TODO - port it to DB
    private static List<String> fpResultList;
    private static List<String> gcList; 

    //Last time the GC thread was run
    private static long gcTimestamp;
    private static long gcInterval;

    static {
        fpResultMap = new HashMap<String, String>();
        fpResultList = new ArrayList<String>();
        gcList = new ArrayList<String>();
     
        gcTimestamp = System.currentTimeMillis();
        gcInterval = PropertyLoader.getInstance(PROPERTIES_FILE).
            getLongValue(GC_INTERVAL) * 1000L;
    }

    /**
     * Singleton instance.
     */
    private FlightPricingState() {
    }

    private synchronized void doGC() {
        //Check if the gc thread have to be started
        long ts = System.currentTimeMillis();
        if ((ts - gcTimestamp) <= gcInterval) {
            return;
        }
        gcTimestamp = ts;
        
        final List<String> gcFinalList = gcList;
        gcList = fpResultList;
        fpResultList = new ArrayList<String>();
        
        Thread gcThread = new Thread() {
            public void run() {
                for (Iterator<String> iter = gcFinalList.iterator(); iter.hasNext();) {
                    fpResultMap.remove(iter.next());
                }
            }
        };
        gcThread.start();
    }

    public static synchronized FlightPricingState getInstance() {
        if (instance == null) {
            instance = new FlightPricingState();
        }
        return instance;
    }
    
    public synchronized void updateResult(IFlightPricingResult fpr) {
        updateResult(fpr.getHeader(), FlightPricingUtils.dumpResult(fpr));
    }
    
    public synchronized void updateResult(String fpr) {
        IHeaderResult hdr = FlightPricingUtils.parseResultHeader(fpr);
        updateResult(hdr, fpr);
    }
    
    private synchronized void updateResult(IHeaderResult hdr, String fpr) {
        doGC(); //Start GC before updating the result
        String uniqueId = createUniqueId(hdr);
        if (fpResultMap.get(uniqueId) == null) {
            fpResultList.add(uniqueId);
        }
        fpResultMap.put(uniqueId, fpr);
    }
    
    public boolean hasResult(String customerId, String queryId) {
        return (fpResultMap.get(createUniqueId(customerId, queryId)) != null);
    }

    public String getResultXml(String customerId, String queryId) {
        return fpResultMap.get(createUniqueId(customerId, queryId));
    }
    
    public IFlightPricingResult getResult(String customerId, String queryId) {
        IFlightPricingResult fpr = null;
        String result = fpResultMap.get(createUniqueId(customerId, queryId));
        if (result != null) {
            try {
                fpr = FlightPricingUtils.parseResult(result);
            }
            catch (IOException ioe) {
                ioe.printStackTrace(); //TODO - log the error
            }
        }
        return fpr;
    }
    
    private String createUniqueId(IHeaderResult hdr) {
        return createUniqueId(hdr.getCustomerId(), hdr.getQueryId());
    }
    
    private String createUniqueId(String customerId, String queryId) {
        return "{" + customerId + ", " + queryId + "}";
    }
}
