package edu.gatech.cercs.soa.qps;

import java.io.*;
import java.sql.*;
import java.util.HashMap;
import java.util.Random;

import edu.gatech.cercs.soa.core.*;
import edu.gatech.cercs.soa.core.fpq.IFlightPricingQuery;
import edu.gatech.cercs.soa.core.fpq.ITrip;
import edu.gatech.cercs.soa.core.fpr.*;


/**
 * Describe <code>QueryProcessor</code> here.
 *
 * @author <a href="mailto:bigpeteb@cc.gatech.edu">Peter Budny</a>
 * @author <a href="mailto:srihari@cc.gatech.edu">Hari Govindharaj</a>
 */
public class QueryProcessor {

    private static final String JDBC_DRIVER = "jdbc.Driver";
    private static final String JDBC_URL = "jdbc.Url";
    private static final String JDBC_USER = "jdbc.User";
    private static final String JDBC_PASSWORD = "jdbc.Password";
    
    private static final String PROPERTIES_FILE = "qps.properties";
    
    private static String dbUrl;
    private static String dbUser;
    private static String dbPasswd;
    
    private static Random rnd;
    
    static {
        PropertyLoader pl = PropertyLoader.getInstance(PROPERTIES_FILE);
        dbUrl = pl.getValue(JDBC_URL);
        dbUser = pl.getValue(JDBC_USER);
        dbPasswd = pl.getValue(JDBC_PASSWORD);
        
        try {
            //Register the JDBC driver.
            Class.forName(pl.getValue(JDBC_DRIVER));
        }
        catch (ClassNotFoundException cnfe) {
            cnfe.printStackTrace(); //TODO - log the error
        }
        
        rnd = new Random(); //For creating random flight numbers
    }
    
    public String process(String queryXml) {
        IFlightPricingResult fpResult = null;
        IFlightPricingQuery fpQuery = null;        
        try {
            //Parse the query xml string
            fpQuery = FlightPricingUtils.parseQuery(queryXml);
            
            //Search the database
            fpResult = searchFlightPrices(fpQuery);
        }
        catch (IOException ioe) {
            ioe.printStackTrace(); //TODO - log the error
            fpResult = FlightPricingFactory.createErrorResult(
                    ioe.getMessage());
        }
        //Dump the flight pricing result as xml
        return FlightPricingUtils.dumpResult(fpResult);
    }
    
    private IFlightPricingResult searchFlightPrices(
            IFlightPricingQuery fpQuery) {
        // We'll use start+travel for one-way and round-trips, and
        // start+travel+end for open-jaw
        String city_start, city_travel, city_end = null;
        boolean roundtrip = false;
        // if city_end==null and rountrip==false, it is one-way
        // if city_end==null and rountrip==true, it is round-trip
        // if city_end!=null, it is open-jaw

        // decide if this is one-way or round-trip
        if (fpQuery.getItinerary().getTrips().size() == 1) {
            // must be one-way
           ITrip trip = fpQuery.getItinerary().getTrips().get(0);
           city_start = trip.getOrigin();
           city_travel = trip.getDestination();
        }
        else if (fpQuery.getItinerary().getTrips().size() == 2) {
            // round-trip or open-jaw
            if (fpQuery.getItinerary().getTrips().get(0).getDestination().equals(
                fpQuery.getItinerary().getTrips().get(1).getOrigin())) {
               ITrip trip = fpQuery.getItinerary().getTrips().get(0);
               city_start = trip.getOrigin();
               city_travel = trip.getDestination();
               roundtrip = true;
            }
            else {
                // open-jaw... not currently supported, bail
                return FlightPricingFactory.createErrorResult("Open-jaw flights are not supported at this time.");
            }
        }
        else {
            // more than 2 trips? don't understand... bail
            return FlightPricingFactory.createErrorResult("Queries with more than 2 trips are not supported at this time.");
        }


        // create possible searches for round-trip: A-B-A, or A-B + B-A

        // TODO: refactor this code, reduce repetition?

        String sql;
        IFlightPricingResult fpResult = FlightPricingFactory.createResult();
        int count = 0;
        if (city_end == null && roundtrip == false) { // one-way
            sql = "SELECT * FROM fares WHERE (city_orig = '" +
                city_start +
                "' AND city_dest = '" +
                city_travel +
                "' AND (oneway_roundtrip = 1 OR oneway_roundtrip = 3)) OR (city_orig = '" +
                city_travel +
                "' AND city_dest = '" +
                city_start +
                "' AND oneway_roundtrip = 1) ORDER BY base_price";

            Connection conn = null;
            Statement stmt = null; //TODO - use PreparedStatement for optimization
            ResultSet rs = null;
            try {
                conn = DriverManager.getConnection(dbUrl, dbUser, dbPasswd);
                stmt = conn.createStatement();
                rs = stmt.executeQuery(sql);

                HashMap<String, IItineraryResult> ir_map = new HashMap<String, IItineraryResult>();            
                while (rs.next()) {
                    IItineraryResult ir = fpResult.createItinerary();
                    IPriceResult pr = ir.getPrice();
                    pr.setFare(rs.getFloat("base_price"));

                    String carrier_fareclass = rs.getString("carrier") +
                        rs.getString("fare_class").substring(0, 1);
                    IItineraryResult prev_best = (IItineraryResult)(ir_map.get(carrier_fareclass));
                    if(prev_best == null || prev_best.getPrice().getFare() > pr.getFare()) {
                        ITripResult tr = ir.createTrip();
                        IStopResult sr = tr.createStop();
                        sr.setAirline(rs.getString("carrier"));
                        sr.setFlightNumber(Integer.toString(rnd.nextInt(1000)+1));
                        sr.setOrigin(rs.getString("city_orig"));
                        sr.setDestination(rs.getString("city_dest"));
                        sr.setDeparture(DateUtils.parseDefault(rs.getString("date_start")));
                        sr.setArrival(DateUtils.parseDefault(rs.getString("date_end")));
                        sr.setFareClass(rs.getString("fare_class"));
                        tr.addStop(sr);
                        ir.addTrip(tr);

                        ir_map.remove(carrier_fareclass);
                        ir_map.put(carrier_fareclass, ir);
                    }
                }
                for (Object ir : ir_map.values()) {
                    fpResult.addItinerary((IItineraryResult)ir);
                    count++;
                }
            }
            catch (SQLException sqe) {
                sqe.printStackTrace();
                fpResult.getHeader().setStatus(IHeaderResult.STATUS_ERROR);
                fpResult.getHeader().setStatusDetail(sqe.getMessage());
            }
            finally {
                try { if (rs != null) rs.close(); } catch (SQLException sqe) {}
                try { if (stmt != null) stmt.close(); } catch (SQLException sqe) {}
                try { if (conn != null) conn.close(); } catch (SQLException sqe) {}
            }
        }
        else if (city_end == null && roundtrip == true ) {
            sql = "SELECT * FROM fares WHERE (city_orig = '" +
                city_start +
                "' AND city_dest = '" +
                city_travel +
                "' AND oneway_roundtrip = 2) ORDER BY base_price";

            Connection conn = null;
            Statement stmt = null; //TODO - use PreparedStatement for optimization
            ResultSet rs = null;
            try {
                conn = DriverManager.getConnection(dbUrl, dbUser, dbPasswd);
                stmt = conn.createStatement();
                rs = stmt.executeQuery(sql);

                HashMap<String, IItineraryResult> ir_map = new HashMap<String, IItineraryResult>();            
                while (rs.next()) {
                    IItineraryResult ir = fpResult.createItinerary();
                    IPriceResult pr = ir.getPrice();
                    pr.setFare(rs.getFloat("base_price"));

                    String carrier_fareclass = rs.getString("carrier") +
                        rs.getString("fare_class").substring(0, 1);
                    IItineraryResult prev_best = (IItineraryResult)(ir_map.get(carrier_fareclass));
                    if(prev_best == null || prev_best.getPrice().getFare() > pr.getFare()) {
                        ITripResult tr1 = ir.createTrip();
                        ITripResult tr2 = ir.createTrip();
                        IStopResult sr1 = tr1.createStop();
                        sr1.setAirline(rs.getString("carrier"));
                        sr1.setFlightNumber(Integer.toString(rnd.nextInt(1000)+1));
                        sr1.setOrigin(rs.getString("city_orig"));
                        sr1.setDestination(rs.getString("city_dest"));
                        sr1.setDeparture(DateUtils.parseDefault(rs.getString("date_start")));
                        sr1.setArrival(DateUtils.parseDefault(rs.getString("date_end")));
                        sr1.setFareClass(rs.getString("fare_class"));
                        IStopResult sr2 = tr2.createStop();
                        sr2.setAirline(rs.getString("carrier"));
                        sr2.setFlightNumber(Integer.toString(rnd.nextInt(1000+1)));
                        sr2.setOrigin(rs.getString("city_dest"));
                        sr2.setDestination(rs.getString("city_orig"));
                        sr2.setDeparture(DateUtils.parseDefault(rs.getString("date_start")));
                        sr2.setArrival(DateUtils.parseDefault(rs.getString("date_end")));
                        sr2.setFareClass(rs.getString("fare_class"));
                        tr1.addStop(sr1);
                        tr2.addStop(sr2);
                        ir.addTrip(tr1);
                        ir.addTrip(tr2);

                        ir_map.remove(carrier_fareclass);
                        ir_map.put(carrier_fareclass, ir);
                    }
                }
                for (Object ir : ir_map.values()) {
                    fpResult.addItinerary((IItineraryResult)ir);
                    count++;
                }
            }
            catch (SQLException sqe) {
                sqe.printStackTrace();
                fpResult.getHeader().setStatus(IHeaderResult.STATUS_ERROR);
                fpResult.getHeader().setStatusDetail(sqe.getMessage());
            }
            finally {
                try { if (rs != null) rs.close(); } catch (SQLException sqe) {}
                try { if (stmt != null) stmt.close(); } catch (SQLException sqe) {}
                try { if (conn != null) conn.close(); } catch (SQLException sqe) {}
            }
        }
        else {
            // currently we shouldn't be able to reach this
            sql = "SELECT 1";
        }

        //Update the result header with the query header info
        FlightPricingUtils.updateResultHeader(fpQuery, fpResult);
        fpResult.getHeader().setStatus(IHeaderResult.STATUS_COMPLETE);
        fpResult.getHeader().setStatusDetail("Found " + count + " matching itineraries.");
        return fpResult;
    }
    
    /**
     * Unit testing code
     * @param args
     */
    public static void main(String[] args) throws Exception {
        if (args.length != 1) {
            System.out.println("Usage: java QueryProcessor <query-file>");
            return;
        }
        
        StringBuffer queryXml = new StringBuffer();
        BufferedReader br = new BufferedReader(new FileReader(args[0]));
        String line = null;
        while ((line = br.readLine()) != null) queryXml.append(line);
        br.close();
        
        QueryProcessor qp = new QueryProcessor();
        System.out.println("Result:\n" + qp.process(queryXml.toString()));
    }
}
